//
//  LowLightWrapper.h
//  ITCCHDR
//
//  Created by ittiam on 11/25/15.
//  Copyright © 2015 Cocologics. All rights reserved.
//

#import <CoreFoundation/CoreFoundation.h>
@import UIKit;
@import CoreVideo;
#import <ITCCHDR/ITCCCommonUtils.h>
#import <ITCCHDR/itt_ll_interface.h>

@class CCYUVImage;
@protocol ITCCLowLightDelegate;

typedef NS_ENUM(NSInteger, ITCCAddImageStatus) {
    ITCCAddImageSuccess,
    ITCCAddImageWait,
    ITCCAddImageLimitExceeded,
    ITCCAddImageFailed
};

@interface ITCCLowLightWrapper : NSObject

@property (nonatomic, weak) id <ITCCLowLightDelegate> delegate;

@property (nonatomic, assign) ITT_LL_CAMERA_MODEL_T cameraModel;
@property (nonatomic, assign) uint32_t numCores;
@property (nonatomic, assign) BOOL isGPUAvailable;
@property (nonatomic, readonly) NSInteger numberOfImagesAdded;

@property (assign, readonly) BOOL lowlightProcessInProgress;

+(ITCCLowLightWrapper *) sharedInstance;

/**
 This enables application to do a initialization after the capture of the first image.
 Once the flag is set, to release memory of the buffer pool, 'reset' has to be called (currently only internally within this class).
 Note, that ITTIAM_LOWLIGHT_API.itt_ll_delete needs to be called in order to free all memory.
 */
-(BOOL)isObjInitialized;

/**
 This initializes the algorithm and allocates required internal buffers.
 This function must be called before adding the first image
 
 @param size Resolution of images used.
 @param imageOrientation Orientation of the images
 @param algoMode Selects mode between long exposure and noise reduction mode
 @param maxNumOfImages The maximum number of images that the application will send. 
 The number of images added to the algorithm cannot exceed this number. 
 The number must be lesser than ITT_LL_MAX_NUM_IMAGES
 @param usExposureTime Exposure Time in micro seconds
 @param iso_val iso rating of the image
 @param format Format of the images supplied.
 @param previewSize Size for image buffer to process preview images
 */
- (BOOL)initLLAlgowithSize:(CGSize)size
          imageOrientation:(UIImageOrientation)imageOrientation
               aspectRatio:(float) aspectRatio
                      mode:(ITT_LL_IP_SELECTION_T)algoMode
            maxNumOfImages:(int) maxNumOfImages
              exposureTime:(int) usExposureTime
                       ISO:(int) iso_val
                    format:(UInt32) format
               previewSize:(CGSize)previewSize;

/** 
 Returns size which has been used to initialize the algorithm. If isObjInitialized returns NO, this method will return CGSizeZero
 */
- (CGSize)currentSize;

/** 
 Returns previewSize which has been used to initialize the algorithm. If isObjInitialized returns NO, this method will return CGSizeZero
 */
- (CGSize)currentPreviewSize;

- (ITCCAddImageStatus)addImageToLib:(CVPixelBufferRef)imageBuffer;

- (BOOL)processImageWithBrightnessScale:(float)brightnessScale
                          isFinalOutput:(BOOL)finalImageFlag;

- (BOOL)processPreviewImageWithBrightnessScale:(float)brightnessScale;

- (void)cancelProcess;

- (CCYUVImage *)cropAndConvertYUVSPImage:(CCYUVImage *)yuvspImage
                                  stride:(uint32_t)stride
                           toAspectRatio:(float)aspectRatio
                      withinCroppingRect:(CGRect)croppingRect
                        imageOrientation:(UIImageOrientation)imageOrientation;

/**
 Image buffers for processing are created automatically (ImageFormat YUV420SP) on property access, suitable for use as the outBuffer in
 processImagesForOutputBuffer.
 The buffer is created manually, _not_ from a CVPixelBufferPool. You need to release it by calling releaseImageBuffersForProcessing!
 The full size and preview size buffers are relased at the same time.
 */
- (void)releaseImageBuffersForProcessing;

@end

@protocol ITCCLowLightDelegate <NSObject>

- (void)lowLightProcessingCompletedWithOutput:(CCYUVImage *)output
                                       stride:(uint32_t)bytesPerRow
                                  aspectRatio:(float)aspectRatio
                                croppedToRect:(CGRect)croppingRect
                               withBrightness:(float)brightnessScale
                                isFinalOutput:(BOOL)finalOutputFlag
                                        error:(ITT_LL_ERR_TYPE_T)error;

- (void)showPreviewImage:(CVPixelBufferRef)previewImage
             orientation:(UIImageOrientation)orientation;


@end
